<?php

/*
Plugin Name: Category class
Plugin URI: http://www.duy-pham.fr
Description: This is just a class for developpers which can be used into plugins, widget, ...
Author: LordPretender
Version: 1.1
Author URI: http://www.duy-pham.fr
*/

//NOT EDITABLE
define ('CCS_DIR', rtrim(plugin_dir_path(__FILE__), '/') );
define ('DEFAULT_NBPOSTS', 30);

require( CCS_DIR . '/inc/squelette.class.php');
require( CCS_DIR . '/inc/categorie.class.php');
require( CCS_DIR . '/inc/article.class.php');

class CategoryClassSingleton {
	/**
	* @var Singleton
	* @access private
	* @static
	*/
	private static $_instance = null;
	
	/**
	* Instance de la catégorie de départ.
	* 
	* @var CCS_Catégorie
	* @access private
	* @static
	*/
	private static $root = null;
	
	/**
	* Numéro de la catégorie de départ.
	* 
	* @var int
	* @access private
	* @static
	*/
	private static $categoryID = -1;
	
	/**
	* Tableau des catégories WP
	* @var Array
	* @access private
	* @static
	*/
	private static $catégories = null;
	
	/**
	* Tableau associatif qui regroupe tous les précédents chargement de catégories en fonction des paramètres fournis.
	* 
	* @var Array
	* @access private
	* @static
	*/
	private static $history = null;
	
	/**
	* Before doing anything with this plugin, you MUST call this method in order to load informations.
	* 
	* @param int $catID					Starting category ID.
	* @param bool $withSubCategories	Load subcategories ?
	* @param bool $withPosts			Load posts ?
	* @param int $nbPosts				Number of posts to load.
	* @param bool $withDuplicatePosts	Having the same post in several categories is authorized ?
	* @param string $orderby			Order by clause for posts.
	* 
	*/
	public static function CCS_loadCategory($catID = 0, $withSubCategories = TRUE, $withPosts = FALSE, $nbPosts = -1, $withDuplicatePosts = FALSE, $orderby = "DESC"){
		self::getInstance();
		
		//On s'assure de la conformité de l'ID de la catégorie de départ.
		$catID = is_int($catID) && $catID >= 0 ? $catID : 0;
		self::$categoryID = $catID;
		
		//On s'assure de la conformité du nombre d'articles par catégories à charger.
		$nbPosts = is_int($nbPosts) && $nbPosts >= 0 ? $nbPosts : DEFAULT_NBPOSTS;
		
		//Création de l'objet de départ (sauf si déjà lu plus tôt) puis chargement.
		if(array_key_exists($catID.$withSubCategories.$withPosts.$nbPosts.$withDuplicatePosts.$orderby, self::$history)){
			self::$root = self::$history[$catID.$withSubCategories.$withPosts.$nbPosts.$withDuplicatePosts.$orderby];
		}else{
			self::$root = new CCS_Catégorie($catID, $withSubCategories, $withPosts, $nbPosts, $withDuplicatePosts, $orderby);
			
			self::$history[$catID.$withSubCategories.$withPosts.$nbPosts.$withDuplicatePosts.$orderby] = self::$root;
		}
	}
	
	/**
	* This method retrive the current category or the list of categories when we are on a post page.
	* 
	* @return array IDs.
	*/
	public static function CCS_getCurrentCategories(){
		$courant = array();
		
		if(is_category()){ //Nous sommes dans une catégorie
			
			//Lecture de l'ID de la catégorie en cours
			$courant[0] = get_query_var('cat');
			
		}elseif (is_front_page() || is_page() || is_author() || is_tag() || is_search() || is_attachment()){ //Nous sommes sur la page d'accueil
			
			$courant[0] = 0;
			
		}else{ //Nous sommes dans un article
			
			foreach(get_the_category() as $categorie){
				//Lecture des IDs
				$courant[] = $categorie->cat_ID;
			}
			
		}
		
		return $courant;
	}
	
	/**
	* This method return a list of category (if a list of category IDs has been sent when calling this method).
	* The method "CCS_loadCategory" MUST be called before !
	* 
	* @param int/array $IDs List of category IDs
	* @param bool $Recursively search in sub-categories too ? (default : TRUE)
	* @param bool $mainSubCategory search the root sub-category which the category depends ? (default : FALSE)
	* 
	* @return array of CCS_Catégorie
	*/
	public static function CCS_getCategories($IDs = -1, $Recursively = TRUE, $mainSubCategory = FALSE, $forceArray = FALSE){
		$retour = array();
		$un = is_array($IDs) || $forceArray ? FALSE : TRUE;
		
		//En fonction du paramètre fourni, soit un récupère toutes les catégories en cours, soit un construit un tableau à partir de l'ID fourni.
		if($IDs == -1){
			$IDs = self::CCS_getCurrentCategories();
		}else{
			if(is_int($IDs))$IDs = array($IDs);
		}
		
		//On passe en revue toutes les catégories fournies
		foreach($IDs as $ID){
			//On récupère l'objet de la catégorie qui nous intéresse.
			$catégorie = self::$root->getCatégorie($ID, $Recursively, $mainSubCategory);
			
			//Remplissage du tableau qui sera retourné
			if(!is_null($catégorie))$retour[$catégorie->getID()] = $catégorie;
		}
		
		//On enlève les ID des catégories qui ne servaient qu'à s'assurer qu'on n'aurait pas de doublons.
		$retour = array_values($retour);
		
		//Si nous n'avions qu'un seul ID à chercher, alors on ne retournera pas de tableau, mais directement l'objet
		if($un)$retour = $retour[0];
		
		return $retour;
	}
	
	/**
	* This method returns a list of posts (all or only someone and only the beginning category or with all subcategories).
	* The method CCS_loadCategory MUST be called before !!
	* 
	* @param int $nbPosts Number of posts to return (less than 0 if you to get all posts)
	* @param bool $withRecursivity Do you want to get subcategories posts too ?
	* 
	*/
	public static function CCS_getPosts($nbPosts = 10, $withRecursivity = FALSE){
		$posts = array();
		
		if(!is_null(self::$root)){
			//On récupère tous les articles (avec récursivité ou non)
			$posts = self::$root->getArticles($withRecursivity);
			
			//Ensuite, on trie les données
			krsort($posts);
			
			//On récupère les X articles demandés
			$posts = array_values($posts);
			if(intval($nbPosts) > 0)$posts = array_slice($posts, 0, intval($nbPosts));
		}
		
		return $posts;
	}
	
	/**
	* With this method you will be able to change the root category without having to manke any data reloading.
	* 
	* @param CCS_Catégorie $object
	* 
	* @return CCS_Catégorie
	*/
	public static function CCS_setRootCategory($object){
		self::$root = $object;
	}
	
	private function __construct(){
		self::$history = array();
	}
	
	/**
	* Méthode qui crée l'unique instance de la classe
	* si elle n'existe pas encore puis la retourne.
	*
	* @param void
	* @return Singleton
	*/
	public static function getInstance() {
		if(is_null(self::$_instance))self::$_instance = new CategoryClassSingleton();  

		return self::$_instance;
	}

}

?>